////////////////////////////////////////////////////////////////////////////////
//                            TemplateMesh.h                                  //
////////////////////////////////////////////////////////////////////////////////

#ifndef TEMPLATEMESH_H
#define TEMPLATEMESH_H

#include "X3DMeshNode.h"
#include "TemplateDirectEdge.h"
#include "TemplateIndirectEdge.h"

#include <iostream>

namespace X3DTK {
namespace Mesh {

/// Template class providing a mesh structure based upon oriented edges, vertices and faces.
/// The integrity of the structure is ensured by the interface of creation. When a new face is 
/// created, the vertices and edges are updated. 

template<class MData, class VData, class EData, class FData>
class TemplateMesh : public X3DNode
{
public:
  /// Defines the Vertex type for the Mesh.
  typedef TemplateVertex<VData, EData, FData> Vertex;
  /// Defines the MVertex type for the Mesh.
  typedef MTemplateVertex<VData, EData, FData> MVertex;
  /// Defines the Edge type for the Mesh.  
  typedef TemplateEdge<EData, FData, VData> Edge;
  /// Defines the MEdge type for the Mesh.  
  typedef MTemplateEdge<EData, FData, VData> MEdge;
  /// Defines the Face type for the Mesh.  
  typedef TemplateFace<FData, VData, EData> Face;
  /// Defines the MFace type for the Mesh.  
  typedef MTemplateFace<FData, VData, EData> MFace;
  
  /// Constructor.
  TemplateMesh()
  : X3DNode() 
  {
  };
  /// Clones the node.
  virtual SFNode clone() const
  {
    return new TemplateMesh(*this);  
  };
  /// Virtual destructor.
  virtual ~TemplateMesh() 
  {
    // delete vertices
    for (typename MVertex::iterator it = _vertices.begin(); it != _vertices.end(); ++it)
      delete (*it);
      
    // delete edges
    for (typename MEdge::iterator it = _edges.begin(); it != _edges.end(); ++it)
    {
      delete (*it)->_symetric;
      delete (*it)->_edgeContent;
      delete (*it);    
    } 
     
    // delete faces
    for (typename MFace::iterator it = _faces.begin(); it != _faces.end(); ++it)
      delete (*it);    
  };
    
  /// Gets the vertices.
  inline const MVertex &getVertices() const {return _vertices;};
  /// Get the edge from - to.
  Edge *getEdge(Vertex *from, Vertex *to)
  {
    if ((from == 0) || (to == 0))
      return 0;
    
    MEdge from_edges = from->getEdges();
    
    for (typename MEdge::const_iterator it = from_edges.begin(); it != from_edges.end(); ++it)
    {
      if ((*it)->getToVertex() == to) 
        return (*it);
    }
        
    return 0;
  };
  
  /// Gets the edges.
  inline const MEdge &getEdges() const {return _edges;};
  /// Gets the faces.
  inline const MFace &getFaces() const {return _faces;};
  /// Gets the data.  
  inline MData &data() {return _data;};
  /// Gets the data.  
  inline const MData &data() const {return _data;};
    
  /// Creates a new vertex.
  Vertex *createVertex() 
  {
    Vertex *V = new Vertex(_vertices.size());
    _vertices.back() = V;
    _vertices.push_back((Vertex *)(0x1));
        
    return V;
  };
  /// Creates a new vertex at the position i.
  Vertex *createVertex(unsigned int i) 
  {
    if (i > _vertices.size())
    {
      _vertices.back() = 0;
      _vertices.resize(i + 2, 0);
      _vertices[i + 1] = (Vertex *)(0x1);
    }  
    else
    {
      if (_vertices[i] != 0)  
      {
        std::cerr << "TemplateMesh::createVertex: cannot create a Vertex at index " << i << ", a Vertex already exists!" << std::endl;
        return 0;
      }
    }
    
    Vertex *V = new Vertex(i);  
    _vertices[i] = V;
    
    return V;
  };
  /// Creates a new edge.
  Edge *createEdge(Vertex *from, Vertex *to) 
  {
    if ((from == 0) || (to == 0))
      return 0;
   
    TemplateEdgeContent<EData, FData, VData> *content = new TemplateEdgeContent<EData, FData, VData>(from, to);
    Edge *E1 = new TemplateDirectEdge<EData, FData, VData>(content);
    Edge *E2 = new TemplateIndirectEdge<EData, FData, VData>(content);
    
    E1->setSymetric(E2);
    E2->setSymetric(E1);
       
    _edges.back() = E1;
    _edges.push_back((Edge *)(0x1));
    
    // updating content of A and B.
    // to complete... to respect order of edges.
    from->addEdge(E1);
    to->addEdge(E2);
    
    return E1;
  };

  /// Creates a new face from its edges that must be in the good order: edges[0]->getBVertex() == edges[1]->getAVertex()...
  Face *createFace(MEdge &edges) 
  {
    Face *F = new Face(_faces.size(), edges);
    _faces.back() = F;
    _faces.push_back((Face *)(0x1));
    
    // updating content of the edges.
    for (typename MEdge::const_iterator it = edges.begin(); it != edges.end(); ++it)
    {
      (*it)->addLeftFace(F);
      (*it)->getFromVertex()->addFace(F);
    }  
    
    return F;
  };
  /// Creates a new face at position i.
  Face *createFace(unsigned int i, MEdge &edges) 
  {
    if (i > _faces.size())
    {
      _faces.back() = 0;
      _faces.resize(i + 2, 0);
      _faces[i + 1] = (Face *)(0x1);
    }
    else
    {
      if (_faces[i] != 0)  
      {
        std::cerr << "TemplateMesh::createFace: cannot create a Face at index " << i << ", a Face already exists!" << std::endl;
        return;
      }
    }
      
    Face *F = new Face(i, edges);
    _faces[i] = F;
    
    // updating content of the edges.
    for (typename MEdge::const_iterator it = edges.begin(); it != edges.end(); ++it)
    {
      (*it)->addLeftFace();
      (*it)->getFromVertex()->addFace(F);
    }  
        
    return F;
  };
 
protected:
  /// Copy constructor.
  TemplateMesh(const TemplateMesh &N)
  : X3DNode(N), _vertices(N._vertices), _edges(N._edges), _faces(N._faces), _data(N._data)
  {
  };
  
private:
  MVertex _vertices;
  MEdge _edges;
  MFace _faces;
  MData _data;
};

}
}

#endif
