////////////////////////////////////////////////////////////////////////////////
//                            X3DComposedGeometryNode.h                       //
////////////////////////////////////////////////////////////////////////////////

#ifndef X3DCOMPOSEDGEOMETRYNODE_H
#define X3DCOMPOSEDGEOMETRYNODE_H

#include "X3DTypes.h"
#include "X3DGeometry3DNode.h"

namespace X3DTK {
namespace X3D {

class X3DColorNode;
class X3DCoordinateNode;
class X3DNormalNode;
class X3DTextureCoordinateNode;

/// Base class of the <a href="../../doc/X3DSpecs/Part01/components/rendering.html">
/// Rendering component</a> defining an X3DComposedGeometryNode
/// (<a href="../../doc/X3DSpecs/Part01/components/rendering.html#X3DComposedGeometryNode">
/// documentation</a>).

class X3DComposedGeometryNode : public X3DGeometry3DNode
{
public:
  /// Constructor.
  X3DComposedGeometryNode();
  /// Constructs an X3DComposedGeometryNode from its attributes.
  X3DComposedGeometryNode(const X3DColorNode *color,
                          const X3DCoordinateNode *coord,
                          const X3DNormalNode *normal,
                          const X3DTextureCoordinateNode *texCoord,
                          SFBool ccw,
                          SFBool colorPerVertex,
                          SFBool normalPerVertex,
                          SFBool solid);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~X3DComposedGeometryNode();
 
  /// Sets the Color attribute.
  void setColor(const X3DColorNode *color);
  /// Sets the Coord attribute.
  void setCoord(const X3DCoordinateNode *coord);
  /// Sets the Normal attribute.
  void setNormal(const X3DNormalNode *normal);
  /// Sets the TexCoord attribute.
  void setTexCoord(const X3DTextureCoordinateNode *texCoord);
  /// Sets the Ccw attribute.
  void setCcw(SFBool ccw);
  /// Sets the ColorPerVertex attribute.
  void setColorPerVertex(SFBool colorPerVertex);
  /// Sets the NormalPerVertex attribute.
  void setNormalPerVertex(SFBool normalPerVertex);
  /// Sets the Solid attribute.
  void setSolid(SFBool solid);

  /// Gets the Color attribute.
  inline SFNode getColor() const {return _color;};
  /// Gets the Coord attribute.
  inline SFNode getCoord() const {return _coord;};
  /// Gets the Normal attribute.
  inline SFNode getNormal() const {return _normal;};
  /// Gets the TexCoord attribute
  inline SFNode getTexCoord() const {return _texCoord;};
  /// Gets the Ccw attribute.
  inline SFBool getCcw() const {return _ccw;};
  /// Gets the ColorPerVertex attribute.
  inline SFBool getColorPerVertex() const {return _colorPerVertex;};
  /// Gets the NormalPerVertex attribute.
  inline SFBool getNormalPerVertex() const {return _normalPerVertex;};
  /// Gets the Solid attribute.
  inline SFBool getSolid() const {return _solid;};
  
  /// Adds the child N. Returns FALSE if the child already exists or N is not a 
  /// valid child.   
  bool addChild(const SFAbstractNode &N);
  /// Sets the child N. Returns FALSE if N is not a 
  /// valid child. If the child already exists then N becomes the new child.
  bool setChild(const SFAbstractNode &N);
  /// Removes the child if exists. Returns TRUE if succeeds.
  bool removeChild(const SFAbstractNode &N);
  
  /// Loads the attributes from a X3DFileElement *.
  virtual void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString write() const; 
 
protected:
  /// Copy constructor.
  X3DComposedGeometryNode(const X3DComposedGeometryNode &N);

private:  
  /// Color attribute.
  SFNode _color;  
  /// Coord attribute.
  SFNode _coord;  
  /// Normal attribute.
  SFNode _normal;
  /// TexCoord attribute.
  SFNode _texCoord;
  /// Ccw attribute.
  SFBool _ccw;
  /// ColorPerVertex attribute.
  SFBool _colorPerVertex;
  /// NormalPerVertex attribute.
  SFBool _normalPerVertex;
  /// Solid attribute.
  SFBool _solid;
};

}
}

#endif
