/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            MeshBuilder.h                                   //
////////////////////////////////////////////////////////////////////////////////

#ifndef MESHBUILDER_H
#define MESHBUILDER_H

#include "X3DTypes.h"
#include "MeshTypes.h"
#include "TemplateVertex.h"
#include "TemplateEdge.h"
#include "TemplateFace.h"
#include "TemplateMesh.h"
#include "TemplateMeshBuilder.h"
#include "IndexedFaceSet.h"
#include "Coordinate.h"
#include "Normal.h"
#include "TextureCoordinate.h"
#include "Color.h"
#include "ColorRGBA.h"

#include <list>
#include <iostream>

namespace X3DTK {
namespace Mesh {

/// Class providing the content of VertexData for one face.

class VertexFData
{
public:
  /// Constructor.
  VertexFData();

  /// Sets the normal.
  void setNormal(const SFVec3f &normal);
  /// Sets the color.
  void setColor(const SFColorRGBA &color);
  /// Sets the texCoord.
  void setTexCoord(const SFVec2f &texCoord);

  /// Gets the normal.
  inline const SFVec3f &getNormal() const {return _normal;};
  /// Gets the color.
  inline const SFColorRGBA &getColor() const {return _color;};
  /// Gets the texCoord.
  inline const SFVec2f &getTexCoord() const {return _texCoord;};  
  
private:
  SFVec3f _normal;
  SFColorRGBA _color;
  SFVec2f _texCoord;         
};

/// Class providing the default content of the template TemplateVertex class.

class VertexData
{
public:
  /// Constructor.
  VertexData();
  /// Destructor.
  ~VertexData();
  
  /// Sets the point.
  void setPoint(const SFVec3f &point);
  /// Adds data related with face.
  void addFaceData(void *face, VertexFData *vdata);
  
  /// Gets the point.
  inline const SFVec3f &getPoint() const {return _point;};
  /// Gets the data related with face.
  VertexFData *getFaceData(void *face) const; 
  
private:
  SFVec3f _point;
  std::list<std::pair<void *, VertexFData *> > _FDataList;
};

/// Class providing the default content of the template TemplateEdge class.

class EdgeData
{
public:
  /// Constructor.
  EdgeData();
};

/// Class providing the default content of the template TemplateFace class.

class FaceData
{
public:
  /// Constructor.
  FaceData();
  
  /// Sets the normal.
  void setNormal(const SFVec3f &normal);
  /// Sets the color.
  void setColor(const SFColorRGBA &color);
  
  /// Gets the normal.
  inline const SFVec3f &getNormal() const {return _normal;};
  /// Gets the color.
  inline const SFColorRGBA &getColor() const {return _color;};
  
private:
  SFVec3f _normal;
  SFColorRGBA _color;
};

/// Class providing the default content of the TemplateMesh class.

class MeshData
{
public:
  /// Constructor.
  MeshData();
  
  /// Sets the boolean vertexNormal, true if vertex has normal.
  void setVertexNormal(bool vertexNormal);
  /// Sets the boolean vertexColor, true if vertex has color.
  void setVertexColor(bool vertexColor);
  /// Sets the boolean vertexTexCoord, true if vertex has normal.
  void setVertexTexCoord(bool vertexTexCoord);
  /// Sets the boolean faceNormal, true if face has normal.
  void setFaceNormal(bool faceNormal);
  /// Sets the boolean faceColor, true if face has color.
  void setFaceColor(bool faceColor);
  
  /// Gets the boolean vertexNormal.
  inline bool hasVertexNormal() const {return _vertexNormal;};
  /// Gets the boolean vertexColor.
  inline bool hasVertexColor() const {return _vertexColor;};
  /// Gets the boolean vertexTexCoord.
  inline bool hasVertexTexCoord() const {return _vertexTexCoord;};
  /// Gets the boolean faceNormal.
  inline bool hasFaceNormal() const {return _faceNormal;};
  /// Gets the boolean faceColor.
  inline bool hasFaceColor() const {return _faceColor;};
  
private:
  bool _vertexNormal; 
  bool _vertexColor; 
  bool _vertexTexCoord;
  bool _faceNormal;
  bool _faceColor;
};

typedef TemplateMesh<MeshData, VertexData, EdgeData, FaceData> Mesh;

/// Constructor defined to name the node.
Mesh::TemplateMesh()
{
  defineTypeName("Mesh");
};


}

namespace X3D {

typedef TemplateMeshBuilder<Mesh::MeshData, Mesh::VertexData, Mesh::EdgeData, Mesh::FaceData> MeshBuilder;

}
}

#endif
