/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            MeshTypes.h                                     //
////////////////////////////////////////////////////////////////////////////////

#ifndef MESHTYPES_H
#define MESHTYPES_H

#include <vector>
#include <list>
#include <iostream>

namespace X3DTK {
namespace Mesh {

class X3DNode;
typedef X3DNode *SFNode;
typedef std::list<SFNode> MFNode;

template<class A, class B, class C> class TemplateVertex;
template<class A, class B, class C> class TemplateEdge;
template<class A, class B, class C> class TemplateFace;

/// Class providing an interface for an object iterator. An object iterator is used for
/// vector of objects, and the next object pointed is not NULL.

template<class T>
class Object_Iterator
{
public:
  /// Default constructor.
  Object_Iterator(): _pointer(0) {};
  /// Explicit constructor from the pointer.
  explicit Object_Iterator(T *pointer): _pointer(pointer) {};
  
  /// Returns the pointed value.
  inline T &operator*() {return static_cast<T &>(*_pointer);};
  /// Returns the pointed value.
  inline const T &operator*() const {return static_cast<T &>(*_pointer);};
  /// Returns the pointer.  
  T *operator->() const {return _pointer;};
  /// Increments the iterator.
  Object_Iterator<T> &operator++() 
  {
    _pointer += 1;
    while (*_pointer == 0)
      _pointer += 1;
      
    return *this;
  };
  /// Increments the iterator.  
  Object_Iterator<T> &operator++(int) 
  {
    Object_Iterator<T> &tmp = *this;
    
    _pointer += 1;
    while (*_pointer == 0)
      _pointer += 1;
      
    return *tmp;
  };
  /// Decrements the iterator.  
  Object_Iterator<T> &operator--() 
  {
    _pointer -= 1;
    while (*_pointer == 0)
      _pointer -= 1;
      
    return *this;
  };
  /// Decrements the iterator.    
  Object_Iterator<T> &operator--(int) 
  {
    Object_Iterator<T> &tmp = *this;
    
    _pointer -= 1;
    while (*_pointer == 0)
      _pointer -= 1;
      
    return *tmp;
  };
  
  friend bool operator==<>(const Object_Iterator<T> &x, const Object_Iterator<T> &y);
  friend bool operator!=<>(const Object_Iterator<T> &x, const Object_Iterator<T> &y);
  
protected:
  /// Pointer.
  T *_pointer;  
};

template<class T>
inline bool operator==(const Object_Iterator<T> &x, const Object_Iterator<T> &y)
{
  return x._pointer == y._pointer;
};

template<class T>
inline bool operator!=(const Object_Iterator<T> &x, const Object_Iterator<T> &y)
{
  return x._pointer != y._pointer;
};

/// Class providing a const object iterator.

template<class T>
class Const_Object_Iterator : public Object_Iterator<const T>
{
public:
  /// Default constructor.
  Const_Object_Iterator() : Object_Iterator<const T>() {};
  /// Constructor from the pointer.
  explicit Const_Object_Iterator(const T *pointer): Object_Iterator<const T>(pointer) {};
};

/// Class providing a non-const object iterator.

template<class T>
class Non_Const_Object_Iterator : public Object_Iterator<T>
{
public:
  /// Default constructor.
  Non_Const_Object_Iterator() : Object_Iterator<T>() {};
  /// Constructor from the pointer.
  explicit Non_Const_Object_Iterator(T *pointer): Object_Iterator<T>(pointer) {};

  /// Conversion operator.
  inline operator Const_Object_Iterator<T>() const 
  {
    return Const_Object_Iterator<T>(_pointer);
  };
};

#ifndef DOXYGEN

template<class T>
struct MBase
{
  typedef Non_Const_Object_Iterator<T> iterator;
  typedef Const_Object_Iterator<T> const_iterator;
};

#endif

/// Class providing a vector of Vertex for convenience, as well as object iterators.

template<class VData, class EData, class FData>
class MTemplateVertex : public std::vector<TemplateVertex<VData, EData, FData> *>, public MBase<TemplateVertex<VData, EData, FData> *>
{
public:
  /// Constructor.
  MTemplateVertex() : std::vector<TemplateVertex<VData, EData, FData> *>(1, (TemplateVertex<VData, EData, FData> *)(0x1)) {};
  /// Constructor from a vector.
  explicit MTemplateVertex(const std::vector<TemplateVertex<VData, EData, FData> *> &vector)
  : std::vector<TemplateVertex<VData, EData, FData> *>(vector)
  {
    push_back((TemplateVertex<VData, EData, FData> *)(0x1));
  }; 
  
  /// Defines an iterator.
  typedef typename MBase<TemplateVertex<VData, EData, FData> *>::iterator iterator;
  /// Defines a const_iterator.
  typedef typename MBase<TemplateVertex<VData, EData, FData> *>::const_iterator const_iterator;
  
  /// Returns an iterator pointing at the beginning of the array.  
  inline iterator begin() {return iterator(&front());};
  /// Returns an iterator pointing at the end of the array.
  inline iterator end() {return iterator(&back());};

  /// Returns an iterator pointing at the beginning of the array.  
  inline const_iterator begin() const {return const_iterator(&front());};
  /// Returns an iterator pointing at the end of the array.
  inline const_iterator end() const {return const_iterator(&back());};
  
  /// to be completed + add member size? not to count the 0 ref!!!!!!!
  inline unsigned int size() const {return static_cast<const std::vector<TemplateVertex<VData, EData, FData> *> *const>(this)->size() - 1;};
  /// Returns true if empty.
  inline bool empty() const {return size() == 1;};
};

/// Class providing a vector of Edge for convenience, as well as object iterators.

template<class EData, class FData, class VData>
class MTemplateEdge : public std::vector<TemplateEdge<EData, FData, VData> *>, public MBase<TemplateEdge<EData, FData, VData> *>
{
public:
  /// Constructor.
  MTemplateEdge() : std::vector<TemplateEdge<EData, FData, VData> *>(1, (TemplateEdge<EData, FData, VData> *)(0x1)) {};
  /// Constructor from a vector.
  explicit MTemplateEdge(const std::vector<TemplateEdge<EData, FData, VData> *> &vector, bool ccw)
  : std::vector<TemplateEdge<EData, FData, VData> *>(vector)
  {
    if (!ccw)
    {
      unsigned int last = vector.size() - 1;
      unsigned int mid = (last + 1)/2;
      for (unsigned int i = 0; i < mid; ++i)
      {
        TemplateEdge<EData, FData, VData> *temp = (*this)[i];
        (*this)[i] = (*this)[last - i];
        (*this)[last - i] = temp;
      }
    }  
         
    push_back((TemplateEdge<EData, FData, VData> *)(0x1));
  };
  /// Defines an iterator.
  typedef typename MBase<TemplateEdge<EData, FData, VData> *>::iterator iterator;
  /// Defines a const_iterator.
  typedef typename MBase<TemplateEdge<EData, FData, VData> *>::const_iterator const_iterator;
  
  /// Returns an iterator pointing at the beginning of the array. 
  inline iterator begin() {return iterator(&front());};
  /// Returns an iterator pointing at the end of the array.
  inline iterator end() {return iterator(&back());};

  /// Returns an iterator pointing at the beginning of the array. 
  inline const_iterator begin() const {return const_iterator(&front());};
  /// Returns an iterator pointing at the end of the array.
  inline const_iterator end() const {return const_iterator(&back());};

  /// Returns the size of the array.
  inline unsigned int size() const {return static_cast<const std::vector<TemplateEdge<EData, FData, VData> *> *const>(this)->size() - 1;};
  /// Returns true if empty.
  inline bool empty() const {return size() == 1;};
};

/// Class providing a vector of Face for convenience, as well as object iterators.

template<class FData, class VData, class EData>
class MTemplateFace : public std::vector<TemplateFace<FData, VData, EData> *>, public MBase<TemplateFace<FData, VData, EData> *>
{
public:
  /// Constructor.
  MTemplateFace() : std::vector<TemplateFace<FData, VData, EData> *>(1, (TemplateFace<FData, VData, EData> *)(0x1)) {};
  /// Constructor from a vector.
  explicit MTemplateFace(const std::vector<TemplateFace<FData, VData, EData> *> &vector)
  : std::vector<TemplateFace<FData, VData, EData> *>(vector)
  {
    push_back((TemplateFace<FData, VData, EData> *)(0x1));
  };
  /// Defines an iterator.
  typedef typename MBase<TemplateFace<FData, VData, EData> *>::iterator iterator;
  /// Defines a const_iterator.
  typedef typename MBase<TemplateFace<FData, VData, EData> *>::const_iterator const_iterator;    
  
  /// Returns an iterator pointing at the beginning of the array. 
  inline iterator begin() {return iterator(&front());};
  /// Returns an iterator pointing at the end of the array.
  inline iterator end() {return iterator(&back());};

  /// Returns an iterator pointing at the beginning of the array.
  inline const_iterator begin() const {return const_iterator(&front());};
  /// Returns an iterator pointing at the end of the array.
  inline const_iterator end() const {return const_iterator(&back());};

  /// Returns the size of the array.
  inline unsigned int size() const {return static_cast<const std::vector<TemplateFace<FData, VData, EData> *> *const>(this)->size() - 1;};
  /// Returns true if empty.
  inline bool empty() const {return size() == 1;};
};

}
}

#endif
